/**
 * Logger for list.
 * @memberof module:apeman-task
 * @constructor ListLogger
 * @returns A logger instance.
 */

"use strict";

var colorprint = require('colorprint'),
    util = require('util'),
    path = require('path'),
    textconv = require('textconv'),
    arraysort = require('arraysort'),
    colors = colorprint.colors;

/** @lends ListLogger */
module.exports = colorprint.define(
    /** @lends ListLogger.prototype */
    {
        /**
         * Log task list.
         * @param {object} tasks - Tasks to show.
         * @param {object[]} childTasks - Child tasks to show.
         */
        logList: function (tasks, childTasks) {
            var NAME_MIN_LENGTH = 24;
            var s = this;
            s.debug('');
            s.debug('  Available tasks:');
            s.debug('');
            Object.keys(tasks)
                .sort(arraysort.stringCompare())
                .forEach(function (name) {
                    var desc = s._descTask(tasks[name]);
                    var comment = desc ? colors.blackBright(['//', desc].join(' ')) : '';
                    s.debug("    %s%s", colors.black.bold(textconv.pad(name, NAME_MIN_LENGTH)), comment);
                });
            s.debug('');
            (childTasks || []).forEach(function (data) {
                var namespace = data.namespace;
                var cwd = data.cwd;
                cwd = (cwd && path.relative(process.cwd(), cwd)) || null;
                var location = (cwd ? '"./' + cwd + '"' : '');
                s.debug(colors.black.italic('    /* Tasks in ' + (location || namespace) + ' */'));
                Object.keys(data.tasks)
                    .sort(arraysort.stringCompare())
                    .forEach(function (name) {
                        var desc = s._descTask(data.tasks[name]);
                        var fullName = [namespace, name].join('#');
                        var comment = desc ? colors.blackBright(['//', desc].join(' ')) : '';
                        s.debug("    %s%s", colors.black.bold(textconv.pad(fullName, NAME_MIN_LENGTH)), comment);
                    });
                s.debug('');
            });
            s.debug('');
            s.debug('');
        },
        _descTask: function (task) {
            var s = this;
            if (Array.isArray(task)) {
                return task
                    .map(function (name) {
                        return s._descTask(name) || ("`" + String(name) + "`");
                    })
                    .map(function (desc) {
                        return textconv.strip(desc.trim(), /\./);
                    }).join(', ');
            }
            var $desc = task.$desc || task.$description;
            if ($desc) {
                return $desc;
            }
            if (typeof(task) === 'function') {
                return util.format('function(){/*...*/}');
            }
            return task.desc || '';
        }
    }
);
