/**
 * @function runTask
 * @memberof module:apeman-task
 * @param {Apemanfile} apemanfile - Apemanfile configuration.
 * @param {string|string[]} names - Names of tasks to run.
 * @param {object} options - Optional settings.
 * @param {number}  [options.depth=0] - Task tree depth.
 */

"use strict";

var async = require('async'),
    assert = require('assert'),
    RunLogger = require('../logging/run_logger'),
    ContextLogger = require('../logging/context_logger'),
    extend = require('extend'),
    argx = require('argx');

/** @lends runTask */
function runTask(apemanfile, names, options, callback) {
    var args = argx(arguments);
    apemanfile = args.shift();
    callback = args.pop('function');
    names = [].concat(args.shift('string|array'));
    options = args.pop('object') || {};

    var depth = Number(options.depth || 0),
        silent = !!options.silent,
        verbose = !!options.verbose;

    assert.ok(apemanfile, "apemanfile is required.");

    async.eachSeries(names, function (name, callback) {
        if (/#/.test(name)) {
            runTask.childTask(apemanfile, name, options, callback);
            return;
        }
        async.waterfall([
            function (callback) {
                apemanfile.find("$tasks", name, callback);
            },
            function (task, callback) {
                var logger = new RunLogger({
                        indent: depth,
                        verbose: verbose,
                        disabled: silent
                    }),
                    startAt = new Date();
                if (options.namespace) {
                    logger.logTaskStarted([options.namespace, name].join('#'), task);
                } else {
                    logger.logTaskStarted(name, task);
                }
                async.eachSeries([].concat(task), function (task, callback) {
                    switch (typeof(task)) {
                        case 'string':
                            runTask.nestedTask(apemanfile, task, options, callback);
                            break;
                        case 'function':
                            task({
                                apemanfile: apemanfile,
                                depth: depth,
                                verbose: verbose,
                                logger: new ContextLogger({
                                    disabled: silent,
                                    depth: depth + 1
                                })
                            }, callback);
                            break;
                        default:
                            callback(new Error('Task must be function or string, bud given: ' + String(task)));
                            break;
                    }
                }, function (err) {
                    if (err) {
                        logger.logTaskFailed(err);
                        process.exit(1);
                    } else {
                        logger.logTaskFinished(new Date() - startAt);
                    }
                    callback(err);
                });
            }
        ], callback);
    }, callback);
}

runTask.nestedTask = function runSubTask(apemanfile, name, options, callback) {
    var args = argx(arguments);
    apemanfile = args.shift();
    callback = args.pop('function');
    name = [].concat(args.shift('string|array'));
    options = args.pop('object') || {};

    var subOptions = extend({}, options);
    subOptions.depth = Number(options.depth || 0) + 1;
    var runTask = require('./run_task');
    runTask(apemanfile, name, subOptions, callback);
};

runTask.childTask = function (apemanfile, name, options, callback) {
    var namepaths = name.split(/#/g);
    var namespace = namepaths.shift();
    if (options.namespace) {
        options.namespace = [options.namespace, namespace].join('#');
    } else {
        options.namespace = namespace;
    }
    var $child = apemanfile.$children[namespace];
    if (!$child) {
        callback(new Error('$child not found with name: ' + name));
    }
    var here = process.cwd(),
        cwd = $child.$cwd || process.cwd();
    async.series([
        function (callback) {
            process.chdir(cwd);
            callback(null);
        },
        function (callback) {
            runTask($child, namepaths.join('#'), options, callback);
        },
        function (callback) {
            process.chdir(here);
            callback(null);
        }
    ], callback);


};
module.exports = runTask;
